#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : FeatureSelection_Subset_V2.py

from sklearn.svm import SVC
from sklearn.feature_selection import SelectFromModel
from sklearn.feature_selection import SelectKBest
from sklearn.feature_selection import SelectFpr
from sklearn.feature_selection import SelectFdr

import numpy as np
from sklearn.svm import SVC
from sklearn.ensemble import AdaBoostClassifier
from sklearn.svm import LinearSVC
from sklearn.feature_selection import RFE
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.ensemble import RandomForestClassifier

# import RFE_redefine
import warnings
from scipy.stats import ttest_ind
from sklearn.feature_selection import mutual_info_classif
import pandas as pd
# 单变量筛选特征
# : f_classif, mutual_info_classif
# 对于分类问题(y离散)，可采用：

def invoking_feature_fuc(fmodels_str,X,y,index,number):
	'''

	:param fmodels_str: 特征选择的名称
	:param X: 特征矩阵
	:param y: 标签
	:param index: 特征名称索引
	:param number: 选择的特征个数
	:return: 返回特征索引和特征名称字典
	'''
	index_dict = {}
	feature_name_dict = {}
	# fmodels_str = ['t_test','SK_mic','SK_fdr','SK_fpr','RFE_SVC','RFE_LR','LinearSVC_L1','LR_L1','DT_gini','RF','AdaBoost','GB']
	# print(fmodels_str)
	for m in fmodels_str:
		# print(m)
		if m == 't_test':
			# print('heoo')
			feature_index, feature_name = student_test(X, y, index)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'SK_mic':
			feature_index, feature_name = SK_mic(X, y,index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'SK_fdr':
			feature_index, feature_name = SK_fdr(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'SK_fpr':
			feature_index, feature_name = SK_fpr(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'RFE_SVC':
			feature_index, feature_name = RFE_SVC(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'RFE_LR':
			feature_index, feature_name = RFE_LR(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'LinearSVC_L1':
			feature_index, feature_name = LinearSVC_L1(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'LR_L1':
			feature_index, feature_name = LR_L1(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'DT_gini':
			feature_index, feature_name = DT_gini(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'RF':
			feature_index, feature_name = RF(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'AdaBoost':
			feature_index, feature_name = AdaBoost(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name
		elif m == 'GB':
			feature_index, feature_name = GB(X, y, index,number)
			index_dict[m] = feature_index
			feature_name_dict[m] = feature_name

	return index_dict,feature_name_dict

def student_test(X,y,feature_index):
	# 返回pvalue值小于等于0.05的特征
	# print('t_test')
	# print(np.where(y == 1)[0])
	# print(list(np.where(y == 1)[0]))
	pos_X = X[list(np.where(y == 1)[0]),:]
	neg_X = X[list(np.where(y == 0)[0]),:]
	t_value, p_value = ttest_ind(pos_X, neg_X)
	pvalue_index = sorted([i for i in list(np.where(p_value[0]<0.05)[0])]) # 默认升序排列

	feature_name = [feature_index[i] for i in list(pvalue_index)]

	return pvalue_index,feature_name

def SK_mic(X,y,feature_index,number):

	# print("feature_index",feature_index)

	model = SelectKBest(mutual_info_classif,k=number)
	scores = model.fit(X,y).scores_
	number = min(sum(scores != 0),number)
	# print("number",number)
	rank = np.argsort(-abs(scores))[:number] # descending order
	# print(rank)
	feature_name = [feature_index[i] for i in list(rank)]

	return rank, feature_name

def SK_fpr(X,y,feature_index,number):
	model = SelectFpr(mutual_info_classif)
	scores = model.fit(X,y).scores_
	number = min(sum(scores != 0),number)

	rank = np.argsort(-abs(scores))[:number] # descending order

	feature_name = [feature_index[i] for i in list(rank)]
	return rank, feature_name

def SK_fdr(X,y,feature_index,number):

	model = SelectFdr(mutual_info_classif)
	scores = model.fit(X,y).scores_
	number = min(sum(scores != 0),number)

	rank = np.argsort(-abs(scores))[:number] # descending order

	feature_name = [feature_index[i] for i in list(rank)]
	return rank, feature_name
#Recursive feature elimination
#循环特征删除
def RFE_SVC(X,y,feature_index,number):
	estimator = LinearSVC()
	rfe = RFE(estimator,step=0.5)
	coef = np.where(rfe.fit(X, y).ranking_ == 1)[0]
	rank = np.argsort(-abs(coef))[:number] # descending order
	feature_name = [feature_index[i] for i in list(rank)]

	return rank,feature_name

def RFE_LR(X,y,feature_index,number):
	estimator = LogisticRegression('l1', solver='liblinear')
	rfe = RFE(estimator,step=0.5)
	coef = np.where(rfe.fit(X, y).ranking_ == 1)[0]
	rank = np.argsort(-abs(coef))[:number] # descending order
	feature_name = [feature_index[i] for i in list(rank)]

	return rank, feature_name

# def RFE_RF()
#
#     estimator = RandomForestClassifier(random_state = random_state)
#     rank = RFE_redefine.RFE_custom(X, y, estimator, number ,step= step)
#
#     feature_name = [feature_index[i] for i in list(rank)]
#     rank_X = X[:,rank]
#
#     return rank_X, feature_name
#
# def RFE_GBDT()
#
#     estimator = GradientBoostingClassifier(random_state=random_state)
#     rank = RFE_redefine.RFE_custom(X, y, estimator, number ,step= step)
#     feature_name = [feature_index[i] for i in list(rank)]
#     rank_X = X[:,rank]
#
#     return rank_X, feature_name
#
# def RFE_Xgboost()
#
#     estimator = xgb.XGBClassifier(random_state=random_state)
#     rank = RFE_redefine.RFE_custom(X, y, estimator, number ,step= step)
#     feature_name = [feature_index[i] for i in list(rank)]
#     rank_X = X[:, rank]
#
#     return rank_X, feature_name


#使用SelectFromModl选择特征

#1、基于L1的特征选择

# def getModelInfo(X,y,feature_index, estimator,number):
# 	selector = SelectFromModel(estimator, max_features=number).fit(X, y)
# 	# logistic_l1_X = selector.fit_transform(X, y)
# 	selected_index = selector.get_support(indices=True)  # get features indices
# 	if hasattr(selector.estimator_,'coef_'):
# 		scores_ = abs(selector.estimator_.coef_[0][selected_index])  # 被选择的特征打分
# 	elif hasattr(selector.estimator_,'feature_importances_'):
# 		scores_ = abs(selector.estimator_.feature_importances_[selected_index])  # 被选择的特征打分
# 	else:
# 		scores_ = []
# 	rank = selected_index[np.argsort(-scores_)]  # 按照降序排列的特征索引
# 	feature_names = [feature_index[i] for i in list(rank)]
# 	return selected_index, feature_names


def getModelInfo(X,y,feature_index, estimator,number):

	selector = estimator.fit(X, y)
	# logistic_l1_X = selector.fit_transform(X, y)
	# selected_index = selector.get_support(indices=True)  # get features indices
	if hasattr(selector,'coef_'):
		scores_ = abs(selector.coef_[0])  # 被选择的特征打分
	elif hasattr(selector,'feature_importances_'):
		scores_ = selector.feature_importances_ # 被选择的特征打分
	else:
		scores_ = []
	rank = np.argsort(-scores_)[0:number]  # 按照降序排列的特征索引
	feature_names = [feature_index[i] for i in list(rank)]
	return rank, feature_names

def LinearSVC_L1(X,y,feature_index,number):
	#https://scikit-learn.org/stable/modules/generated/sklearn.svm.LinearSVC.html#sklearn.svm.LinearSVC
	#This class supports both dense and sparse input
	# and the multiclass support is handled according to a one-vs-the-rest scheme.
	estimator = LinearSVC('l1', loss='squared_hinge', dual=False)
	selected_index, feature_names = getModelInfo(X,y,feature_index, estimator,number)
	return selected_index, feature_names

def LR_L1(X,y,feature_index,number):

	#If the option chosen is ‘ovr’, then a binary problem is.fit_transform for each label.
	estimator = LogisticRegression()
	selected_index, feature_names = getModelInfo(X,y,feature_index, estimator,number)
	return selected_index, feature_names

# 2、基于树模型的特征选择法

# tree 类

def DT_gini(X,y,feature_index,number):

	# http://scikit-learn.org/stable/modules/generated/sklearn.tree.DecisionTreeClassifier.html#sklearn.tree.DecisionTreeClassifier.feature_importances_
	estimator = DecisionTreeClassifier()
	selected_index, feature_names = getModelInfo(X, y, feature_index, estimator, number)
	return selected_index, feature_names


def RF(X,y,feature_index,number):
	# 默认参数：n_estimators=’warn’, criterion=’gini’
	# http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.RandomForestClassifier.html

	estimator = RandomForestClassifier()
	selected_index, feature_names = getModelInfo(X, y, feature_index, estimator, number)
	return selected_index, feature_names


def AdaBoost(X,y,feature_index,number):
	estimator = AdaBoostClassifier()
	selected_index, feature_names = getModelInfo(X, y, feature_index, estimator, number)
	return selected_index, feature_names


def GB(X,y,feature_index,number):
	estimator = GradientBoostingClassifier()
	selected_index, feature_names = getModelInfo(X, y, feature_index, estimator, number)
	return selected_index, feature_names


