#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : get_data.py
# Author: My Duan
# Date  : 2022-09-30

import pandas as pd
from data_loader import *
from options import parse_args
from sklearn.preprocessing import OneHotEncoder
from utils import *


def preprocess_feats(obj_data, regu, target):

	# fill all the missing features and unavaliable features by 0
	# transform omit: feats
	# if exits, extract as regu/targ, else fill by 0
	feats = obj_data.columns.to_list()
	feats = list(np.unique(np.array(feats)))

	if feats[0].find('.') != -1:
		feats = [i.split('.')[0] for i in feats]

	obj_data.columns = feats

	unavaliable_feats = list(np.array(feats)[obj_data.isnull().all().values])

	for k in regu:
		if k not in feats:
			unavaliable_feats.append(k)

	for k in target:
		if k not in feats:
			unavaliable_feats.append(k)

	# print('Here')
	# print(len(unavaliable_feats))

	if len(unavaliable_feats) !=0:
		print('\tUnavaliable features number is {0}. Padding these genes by 0.0.'.format(len(unavaliable_feats)))
		print(unavaliable_feats)
		obj_data.loc[:, unavaliable_feats] = 0.0

	else:
		print('\tAll regulatory and target genes are avaliable.\n')

	return  obj_data


def get_adjency(adj_matrix):

	edge = np.where(abs(adj_matrix) != 0)
	edge_index = np.vstack((edge[0], edge[1]))

	return edge_index


def get_data(obj_data, class_label, driver, target, missing_value = '-9.9658', log_transform=False):

	temp_data = pd.merge(obj_data, class_label, left_index=True, right_index=True, how='left')
	temp_data = temp_data.dropna(subset=['label']) # how='any

	obj_data = temp_data.iloc[:,:-1]

	class_label_org = temp_data.iloc[:,-1]

	print('Dataset classification label distribution is :')
	print(class_label.value_counts())

	enc = OneHotEncoder()
	class_label = class_label.values.reshape(-1,1)
	class_label = enc.fit_transform(class_label).toarray()


	print('Data shape is: {}, label shape is: {}.'.format(obj_data.shape, class_label.shape))

	
	'''transform the null to np.nan'''

	'''log-transform'''

	if log_transform == True:
		import math
		print('Log transform ...')
		try:
			obj_data = obj_data.applymap(lambda x: math.log2(x + 0.001))
		except:
			print('Please check whether the data needs log transform.')

	# print('missing value is :', missing_value)

	obj_data = transform_na(obj_data, missing_value)
	'''by the driver and target genes, fill unavaliable features by 0.0'''
	print('Check and pad the unavaliabel feautres...')
	obj_data = preprocess_feats(obj_data, driver, target)

	return obj_data, class_label, class_label_org



