
## HealththModel for mqTrans
Orchestrating information across tissues via a novel multi-task GAT framework to improve quantitative gene regulation relation modeling for survival analysis  

Maintenance personnel: Fengfeng Zhou, Meiyu Duan,Kewei Li
Email：ffzhou@jlu.edu.cn, dmy235813@163.com, kwbb1997@gmail.com


## Requirement

anaconda/2020.11  
cuda/10.2  
cudnn/7.6.5.32_cuda10.2  
gcc/7.5  
python/3.7  
torch              1.13.1  
torch-cluster      1.6.1+pt113cu117  
torch-geometric    2.2.0  
torch-scatter      2.1.1+pt113cu117  
torch-sparse       0.6.17+pt113cu117  
torch-spline-conv  1.2.2+pt113cu117   
lifelines          0.27.3

## Command

$ python main.py &lt;data folder> &lt;model folder> &lt;output folder> &lt;pretrained>  

*  &lt;data folder>   
Source data folder, each file is csv format.  
data.csv is the transcriptome file, index is the sample IDs, column name is the gene IDs.  
label.csv is the survival label file, index is the sample IDs, columns iname is label.

* &lt;model folder>  
HealthModel.pth: The pretrained HealthModel.  
regulatory_geneIDs.csv: The regulatory genes used in this study.  
target_geneIDs.csv: The target genes used in this study.  
adjacent_matrix.csv: The adjacent matrix of regulatory genes.


* &lt;output folder>   
The output files will be written to this folder.


## Example input execution：
"python ./Get_mqTrans/code/main.py ./data ./Get_mqTrans/model ./output"

The Python environment is 3.7  
The pytorch version is 1.13.1

main.py is the main execution code

## code folder including:

* preprocess.py is the code to process data and label, including deletes the unavaliable data and survival label, splits dataset, KNN imputation and Z-Normalization.

* data_loader.py is the code to load dataset.

* GAT.py is the HealthModel code.
 
* evaluate.py is the code for training and evaluation of multi-task model
 
* mqTrans.py is the code of pipeline of the framework.
 
* options.py is the code for parameter defination.
 
* utils.py is the code to define early stopper class.

## Output after execution:

* test_target.csv: The gene expression value of target genes after Z-Normalization and imputation
* pred_target.csv: The predict gene expression value of target genes 
* mq_target.csv: The predict gene expression value of target genes 

These files will be saved in the output folder.  

## Log

slurm-807348.out is the the output of the example data.  
